import { ExtensionObject } from "node-opcua-extension-object";
import { StructuredTypeSchema } from "node-opcua-factory";
import { ExtensionObjectBuilder } from "./json_basic_encoding_body_functor";
import { NodeIdJSON } from "./json_basic_encoding_decoding_nodeid";
import { Pojo } from "./decode_opcua_json";
export interface ExtensionObjectJSON {
    TypeId: NodeIdJSON;
    Body: Pojo;
}
export declare function makeBody(extensionObject: ExtensionObject, reversible?: boolean): Pojo;
export declare function opcuaJsonDecodeExtensionObjectBody(schema: StructuredTypeSchema, body: Pojo, builder?: ExtensionObjectBuilder): Pojo;
export declare function opcuaJsonEncodeExtensionObjectBody(extensionObject: ExtensionObject | null, reversible?: boolean, namespaceArray?: string[]): ExtensionObjectJSON | Pojo | null;
/**
 *
 * ExtensionObject values shall be encoded as a JSON object with the fields shown:
 *
 * JSON Object - Definition for a ExtensionObject
 * Name	        Description
 * TypeId	    The NodeId of a DataTypeEncoding Node formatted using the rules in 5.4.2.10.
 * Encoding	    The format of the Body field encoded as a JSON number.
 *                 This value is 0 if the body is Structure encoded as a JSON object (see 5.4.6).
 *                 This value is 1 if the body is a ByteString value encoded as a JSON string (see 5.4.2.8).
 *                 This value is 2 if the body is a XmlElement value encoded as a JSON string (see 5.4.2.9).
 *                 This field is omitted if the value is 0.
 * Body	        Body of the ExtensionObject. The type of this field is specified by the Encoding field.
 *              If the Body is empty, the ExtensionObject is NULL and is omitted or encoded as a JSON null.
 *
 * For the non-reversible form, ExtensionObject values shall be encoded as a JSON object containing only
 * the value of the Body field. The TypeId and Encoding fields are dropped.
 */
export declare function opcuaJsonEncodeExtensionObject(extensionObject: ExtensionObject | null, reversible?: boolean, namespaceArray?: string[]): ExtensionObjectJSON | Pojo | null;
export declare function opcuaJsonDecodeExtensionObject(_pojo: ExtensionObjectJSON, builder?: ExtensionObjectBuilder): ExtensionObject | null;
