/**
 * Copyright Sterfive 2020-2022
 */

import { NodeId } from "node-opcua-nodeid";
export interface NodeIdJSON {
    IdType?: 0 | 1 | 2 | 3;
    Id: number | string;
    Namespace?: number | string;
}
/**
 *
 * IdType	   The IdentifierType encoded as a JSON number.
 *             Allowed values are:
 *               0 - UInt32 Identifier encoded as a JSON number.
 *               1 - A String Identifier encoded as a JSON string.
 *               2 - A Guid Identifier.
 *               3 - A ByteString Identifier
 *
 *             This field is omitted for UInt32 identifiers.
 *
 *  Id         The Identifier.
 *             The value of the id field specifies the encoding of this field.
 *
 *  Namespace	The NamespaceIndex for the NodeId.
 *              The field is encoded as a JSON number for the reversible encoding.
 *              The field is omitted if the NamespaceIndex equals 0.
 *              For the non-reversible encoding, the field is the NamespaceUri associated with the NamespaceIndex,
 *              encoded as a JSON string.
 *  A NamespaceIndex of 1 is always encoded as a JSON number.
 */
export declare function opcuaJsonEncodeNodeId(value: NodeId, reversible?: boolean, namespaceArray?: string[]): NodeIdJSON;
export declare function opcuaJsonDecodeNodeId(pojo: NodeIdJSON, namespaceArray?: string[]): NodeId;
