import { NodeId } from "node-opcua-nodeid";
import { IBasicSession } from "node-opcua-pseudo-session";
import { WriterGroupDataTypeOptions, ReaderGroupDataTypeOptions, KeyValuePair } from "node-opcua-types";
import { UANodeProxy } from "./ua_node_proxy";
import { ReaderGroup } from "./reader_group";
import { WriterGroup } from "./writer_group";
declare enum PubSubState {
    Disabled = 0,
    Paused = 1,
    Operational = 2,
    Error = 3
}
declare const BrokerConnectionTransportNodeIds: {
    AuthenticationProfileUri: string;
    ResourceUri: string;
};
export declare class BrokerConnectionTransport extends UANodeProxy<typeof BrokerConnectionTransportNodeIds> {
    protected _getPaths(): typeof BrokerConnectionTransportNodeIds;
    getAuthenticationProfileUri(): Promise<string>;
    getResourceUri(): Promise<string>;
}
declare const DatagramConnectionTransportNodeIds: {};
export declare class DatagramConnectionTransport extends UANodeProxy<typeof DatagramConnectionTransportNodeIds> {
    protected _getPaths(): typeof DatagramConnectionTransportNodeIds;
}
declare const ConnectionNodeIds: {
    PublisherId: string;
    "Status.State": string;
    TransportProfileUri: string;
    TransportSettings: string;
    ConnectionProperties: string;
    Address: string;
    "Address.Url": string;
    "Address.NetworkInterface": string;
};
export declare class Connection extends UANodeProxy<typeof ConnectionNodeIds> {
    protected _getPaths(): typeof ConnectionNodeIds;
    constructor(session: IBasicSession, nodeId: NodeId);
    /**
     *
     */
    addReaderGroup(readerGroup: ReaderGroupDataTypeOptions): Promise<ReaderGroup>;
    /**
     *
     */
    addWriterGroup(writerGroup: WriterGroupDataTypeOptions): Promise<WriterGroup>;
    removeGroup(): Promise<void>;
    getReadGroups(): Promise<ReaderGroup[]>;
    getWriterGroups(): Promise<WriterGroup[]>;
    getPublisherId(): Promise<string>;
    getStatusState(): Promise<PubSubState>;
    getTransportSettings(): Promise<BrokerConnectionTransport | DatagramConnectionTransport>;
    getTransportProfileUri(): Promise<string>;
    getConnectionProperties(): Promise<KeyValuePair[]>;
    getAddressNetworkinterface(): Promise<string>;
    getAddressUrl(): Promise<string>;
}
export {};
