import { NodeId, NodeIdLike } from "node-opcua-nodeid";
import { IBasicSession } from "node-opcua-pseudo-session";
import { DataSetFieldFlags, SimpleAttributeOperand, ContentFilter, PublishedVariableDataTypeOptions } from "node-opcua-types";
import { UANodeProxy } from "./ua_node_proxy";
import { PublishedDataItems, PublishedEvents } from "./published_dataset";
declare const DataSetFolderNodeIds: {};
export declare class DataSetFolder extends UANodeProxy<typeof DataSetFolderNodeIds> {
    protected _getPaths(): Record<string, string>;
    constructor(session: IBasicSession, nodeId?: NodeId);
    /**
     *    [in]  String Name
          [in]  String FieldNameAliases
          [in]  DataSetFieldFlags FieldFlags
          [in]  PublishedVariableDataType VariablesToAdd
          [out] NodeId DataSetNodeId
          [out] ConfigurationVersionDataType ConfigurationVersion
          [out] StatusCode AddResults
     */
    addPublishedDataItems({ name, fieldNameAliases, fieldFlags, variablesToAdd, }: {
        name: string;
        fieldNameAliases?: string[] | null;
        fieldFlags?: DataSetFieldFlags[] | null;
        variablesToAdd?: PublishedVariableDataTypeOptions[] | null;
    }): Promise<PublishedDataItems>;
    /**
     *     [in]  String                       Name
     *     [in]  NodeId                       EventNotifier
     *     [in]  String                       FieldNameAliases
     *     [in]  DataSetFieldFlags            FieldFlags
     *     [in]  SimpleAttributeOperand       SelectedFields
     *     [in]  ContentFilter                Filter
     *     [out] ConfigurationVersionDataType ConfigurationVersion
     *     [out] NodeId                       DataSetNodeId
     */
    addPublishedEvents({ name, eventNotifier, fieldNameAliases, fieldFlags, selectedFields, filter, }: {
        name: string;
        eventNotifier: NodeId;
        fieldNameAliases: string;
        fieldFlags: DataSetFieldFlags;
        selectedFields: SimpleAttributeOperand;
        filter: ContentFilter;
    }): Promise<PublishedEvents>;
    addPublishedDataItemsTemplate(): Promise<void>;
    addPublishedEventsTemplate(): Promise<void>;
    removePublishedDataSet(): Promise<void>;
    addDataSetFolder({ name }: {
        name: string;
    }): Promise<DataSetFolder>;
    removeDataSetFolder({ nodeId }: {
        nodeId: NodeIdLike;
    }): Promise<void>;
    getPublishedDataSetCollection(): Promise<(PublishedDataItems | PublishedEvents)[]>;
    getPublishedDataItemsCollection(): Promise<PublishedDataItems[]>;
    getPublishedEventsCollection(): Promise<PublishedEvents[]>;
    getDataSetFolders(): Promise<DataSetFolder[]>;
}
export {};
